/**
  ******************************************************************************
  * @file    Project\Internal_OSC_Calib\inc\InternOscCalibration.h
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    02-August-2013
  * @brief   Header for InternOscCalibration.h
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2013 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __INTERNOSC_CALIBRATION_H
#define __INTERNOSC_CALIBRATION_H

/* Includes ------------------------------------------------------------------*/
/* Exported types ------------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Comment this line if the LSE clock is not used as the reference frequency */
#define USE_REFERENCE_LSE

#ifdef USE_REFERENCE_LSE

  #define TIM14_COUNTER_PRESCALER  0
  /* The signal in input capture is divided by 8 */
  #define TIM14_IC_DIVIDER         TIM_ICPSC_DIV8

  /* The LSE is divided by 8 => LSE/8 = 32768/8 = 4096 */
  #define REFERENCE_FREQUENCY     (uint32_t) 4096 /* The reference frequency value in Hz */

#else

  /* If reference frequency is not the LSE and lower than 250 Hz (The TIM14 min frequency),
       the TIM14 counter prescaler should be used.
     Otherwise (the reference frequency is greater than 250 Hz),
       the TIM14 counter prescaler should be set to 0

     For example, If reference frequency is 50 Hz, the counter prescaler should be 8. 
     In that case the minimum frequency (if HSI is system clock source) is 16MHz/(8*65535) ~ 31 Hz

     If the reference frequency is lower than 31Hz, the counter prescaler should be
     used regarding the reference frequency in input */

  /* The reference frequency is 50 Hz (the signal in input capture is not divided) */
  #define REFERENCE_FREQUENCY     (uint32_t)50 /* The reference frequency value in Hz */

  #define TIM14_COUNTER_PRESCALER  7 /* TIM14 counter is divided by 7 + 1 = 8 */
  /* The signal in input capture is not divided */
  #define TIM14_IC_DIVIDER         TIM_ICPSC_DIV1

#endif /* USE_REFERENCE_LSE */

#define NUMBER_OF_LOOPS    50

/* Exported macro ------------------------------------------------------------*/
#define ABS_RETURN(x)         (x < 0) ? (-x) : x

/* Exported functions ------------------------------------------------------- */
uint32_t HSI_CalibrateMinError(void);
ErrorStatus HSI_CalibrateFixedError(uint32_t MaxAllowedError, uint32_t* Freq);
void RCC_AdjustCalibrationValue_HSI(uint8_t InternOsc, uint8_t TrimmingValue);
void TIM14_ConfigForCalibrationHSI(void);
uint32_t HSI14_CalibrateMinError(void);
ErrorStatus HSI14_CalibrateFixedError(uint32_t MaxAllowedError, uint32_t* Freq);
void RCC_AdjustCalibrationValue_HSI14(uint8_t InternOsc, uint8_t TrimmingValue);
void TIM14_ConfigForCalibrationHSI14(void);
void SetSysClock_HSI_48MHz(void);
void SetSysClock_HSI_8MHz(void);
void CLK_ConfigForCalibration(void);
void GPIO_ConfigForCalibration(void);

#endif /* __INTERNOSC_CALIBRATION_H */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
